@@ -38,6 +38,8 @@ EffectChain::EffectChain(float aspect_nom, float aspect_denom, ResourcePool *res
 	  output_color_rgba(false),
 	  output_color_ycbcr(false),
 	  dither_effect(NULL),
+	  intermediate_format(GL_RGBA16F),
+	  intermediate_transformation(NO_FRAMEBUFFER_TRANSFORMATION),
 	  num_dither_bits(0),
 	  output_origin(OUTPUT_ORIGIN_BOTTOM_LEFT),
 	  finalized(false),
@@ -331,7 +333,14 @@ void EffectChain::compile_glsl_program(Phase *phase)
 	
 		frag_shader += string("uniform sampler2D tex_") + effect_id + ";\n";
 		frag_shader += string("vec4 ") + effect_id + "(vec2 tc) {\n";
-		frag_shader += "\treturn tex2D(tex_" + string(effect_id) + ", tc);\n";
+		frag_shader += "\tvec4 tmp = tex2D(tex_" + string(effect_id) + ", tc);\n";
+
+		if (intermediate_transformation == SQUARE_ROOT_FRAMEBUFFER_TRANSFORMATION &&
+		    phase->inputs[i]->output_node->output_gamma_curve == GAMMA_LINEAR) {
+			frag_shader += "\ttmp.rgb *= tmp.rgb;\n";
+		}
+
+		frag_shader += "\treturn tmp;\n";
 		frag_shader += "}\n";
 		frag_shader += "\n";
 
@@ -414,6 +423,15 @@ void EffectChain::compile_glsl_program(Phase *phase)
 			frag_shader_outputs.push_back("RGBA");
 		}
 	}
+
+	// If we're bouncing to a temporary texture, signal transformation if desired.
+	if (!phase->output_node->outgoing_links.empty()) {
+		if (intermediate_transformation == SQUARE_ROOT_FRAMEBUFFER_TRANSFORMATION &&
+		    phase->output_node->output_gamma_curve == GAMMA_LINEAR) {
+			frag_shader += "#define SQUARE_ROOT_TRANSFORMATION 1\n";
+		}
+	}
+
 	frag_shader.append(read_file("footer.frag"));
 
 	// Collect uniforms from all effects and output them. Note that this needs
@@ -650,9 +668,8 @@ Phase *EffectChain::construct_phase(Node *output, map<Node *, Phase *> *complete
 	// Actually make the shader for this phase.
 	compile_glsl_program(phase);
 
-	// Initialize timer objects.
+	// Initialize timers.
 	if (movit_timer_queries_supported) {
-		glGenQueries(1, &phase->timer_query_object);
 		phase->time_elapsed_ns = 0;
 		phase->num_measured_iterations = 0;
 	}
@@ -1176,6 +1193,11 @@ void EffectChain::propagate_alpha()
 
 		if (alpha_handling == Effect::INPUT_AND_OUTPUT_PREMULTIPLIED_ALPHA ||
 		    alpha_handling == Effect::INPUT_PREMULTIPLIED_ALPHA_KEEP_BLANK) {
+			// This combination (requiring premultiplied alpha, but _not_ requiring
+			// linear light) is illegal, since the combination of premultiplied alpha
+			// and nonlinear inputs is meaningless.
+			assert(node->effect->needs_linear_light());
+
 			// If the effect has asked for premultiplied alpha, check that it has got it.
 			if (any_postmultiplied) {
 				node->output_alpha_type = ALPHA_INVALID;
@@ -1693,6 +1715,8 @@ void EffectChain::render_to_fbo(GLuint dest_fbo, unsigned width, unsigned height
 	check_error();
 	glDisable(GL_DITHER);
 	check_error();
+	glEnable(GL_FRAMEBUFFER_SRGB);
+	check_error();
 
 	// Save original viewport.
 	GLuint x = 0, y = 0;
@@ -1736,7 +1760,15 @@ void EffectChain::render_to_fbo(GLuint dest_fbo, unsigned width, unsigned height
 		Phase *phase = phases[phase_num];
 
 		if (do_phase_timing) {
-			glBeginQuery(GL_TIME_ELAPSED, phase->timer_query_object);
+			GLuint timer_query_object;
+			if (phase->timer_query_objects_free.empty()) {
+				glGenQueries(1, &timer_query_object);
+			} else {
+				timer_query_object = phase->timer_query_objects_free.front();
+				phase->timer_query_objects_free.pop_front();
+			}
+			glBeginQuery(GL_TIME_ELAPSED, timer_query_object);
+			phase->timer_query_objects_running.push_back(timer_query_object);
 		}
 		if (phase_num == phases.size() - 1) {
 			// Last phase goes to the output the user specified.
@@ -1778,14 +1810,22 @@ void EffectChain::render_to_fbo(GLuint dest_fbo, unsigned width, unsigned height
 		// Get back the timer queries.
 		for (unsigned phase_num = 0; phase_num < phases.size(); ++phase_num) {
 			Phase *phase = phases[phase_num];
-			GLint available = 0;
-			while (!available) {
-				glGetQueryObjectiv(phase->timer_query_object, GL_QUERY_RESULT_AVAILABLE, &available);
+			for (std::list<GLuint>::iterator timer_it = phase->timer_query_objects_running.begin();
+			     timer_it != phase->timer_query_objects_running.end(); ) {
+				GLint timer_query_object = *timer_it;
+				GLint available;
+				glGetQueryObjectiv(timer_query_object, GL_QUERY_RESULT_AVAILABLE, &available);
+				if (available) {
+					GLuint64 time_elapsed;
+					glGetQueryObjectui64v(timer_query_object, GL_QUERY_RESULT, &time_elapsed);
+					phase->time_elapsed_ns += time_elapsed;
+					++phase->num_measured_iterations;
+					phase->timer_query_objects_free.push_back(timer_query_object);
+					phase->timer_query_objects_running.erase(timer_it++);
+				} else {
+					++timer_it;
+				}
 			}
-			GLuint64 time_elapsed;
-			glGetQueryObjectui64v(phase->timer_query_object, GL_QUERY_RESULT, &time_elapsed);
-			phase->time_elapsed_ns += time_elapsed;
-			++phase->num_measured_iterations;
 		}
 	}
 }
@@ -1838,13 +1878,10 @@ void EffectChain::execute_phase(Phase *phase, bool last_phase,
 	if (!last_phase) {
 		find_output_size(phase);
 
-		GLuint tex_num = resource_pool->create_2d_texture(GL_RGBA16F, phase->output_width, phase->output_height);
+		GLuint tex_num = resource_pool->create_2d_texture(intermediate_format, phase->output_width, phase->output_height);
 		output_textures->insert(make_pair(phase, tex_num));
 	}
 
-	glUseProgram(phase->glsl_program_num);
-	check_error();
-
 	// Set up RTT inputs for this phase.
 	for (unsigned sampler = 0; sampler < phase->inputs.size(); ++sampler) {
 		glActiveTexture(GL_TEXTURE0 + sampler);
@@ -1868,12 +1905,15 @@ void EffectChain::execute_phase(Phase *phase, bool last_phase,
 		glViewport(0, 0, phase->output_width, phase->output_height);
 	}
 
+	GLuint instance_program_num = resource_pool->use_glsl_program(phase->glsl_program_num);
+	check_error();
+
 	// Give the required parameters to all the effects.
 	unsigned sampler_num = phase->inputs.size();
 	for (unsigned i = 0; i < phase->effects.size(); ++i) {
 		Node *node = phase->effects[i];
 		unsigned old_sampler_num = sampler_num;
-		node->effect->set_gl_state(phase->glsl_program_num, phase->effect_ids[node], &sampler_num);
+		node->effect->set_gl_state(instance_program_num, phase->effect_ids[node], &sampler_num);
 		check_error();
 
 		if (node->effect->is_single_texture()) {
@@ -1921,6 +1961,8 @@ void EffectChain::execute_phase(Phase *phase, bool last_phase,
 		node->effect->clear_gl_state();
 	}
 
+	resource_pool->unuse_glsl_program(instance_program_num);
+
 	if (!last_phase) {
 		resource_pool->release_fbo(fbo);
 	}
