@@ -19,13 +19,16 @@ public:
 
 	virtual bool needs_srgb_primaries() const { return false; }
 	virtual AlphaHandling alpha_handling() const { return DONT_CARE_ALPHA_TYPE; }
+	virtual bool one_to_one_sampling() const { return true; }
 
 	virtual void inform_input_size(unsigned input_num, unsigned width, unsigned height);
 	void set_gl_state(GLuint glsl_program_num, const std::string &prefix, unsigned *sampler_num);
 
 private:
-	Point2D center, aspect_correction;
+	Point2D center;
+	Point2D uniform_aspect_correction, uniform_flipped_center;
 	float radius, inner_radius;
+	float uniform_pihalf_div_radius;
 };
 
 }  // namespace movit
